module Echo
  class Backplane < API
    include HTTParty
    headers 'Content-Type' => 'application/x-www-urlencoded'    
    
    def initialize(channel, options={})            
      self.class.debug_output $stdout if Echo.debug
      self.class.base_uri "http://api.echoenabled.com/v1/"
      self.class.basic_auth Echo.backplane_user, Echo.backplane_password
      
      @channel = channel
    end

    # Log in the user in Backplane
    #
    # @params [Hash] the configuration values that should be send to
    # the echo server
    # config = {
    #   :identityUrl => "http://conferize.com/u/Pdegallego",
    #   :source      => "http://conferize.com",
    #   :username    => "Pdelgallego",
    #   :displayName => "Pdelgallego"
    # }
    def login(config, echo_id)      
      data = backplane_login_data(config, echo_id, "identity/login")
      perform_request(data)
    end
    
    # Log out the user in Backplane
    #
    # @params [Hash] the configuration values that should be send to
    # the echo server
    # config = {
    #   :identityUrl => "http://conferize.com/u/Pdegallego",
    #   :source      => "http://conferize.com",
    #   :username    => "Pdelgallego",
    #   :displayName => "Pdelgallego"
    # }
    def logout(config, echo_id)      
      data = backplane_login_data(config, echo_id, "identity/logout")
      perform_request(data)
    end    
    
    private
    def perform_request(data)      
      JSON.parse(`curl -u #{Echo.backplane_user}:#{Echo.backplane_password} --data-binary '#{data}' #{@channel}`)
    end
    
    # Create the json structure that Echo will consume. 
    #
    # @params [String] Each user has associated an echo id that is
    # the one that should be use to login/logout
    #
    # @param [String] The action indicates if the user is logging in
    # or loogging out
    def backplane_login_data(config, echo_id, action)    
      data = [{"type" => action, "source" => config[:source],"payload"=>{"context"=>config[:source],"identities"=>{"entry"=>{"accounts"=>[{"identityUrl" => config[:identityUrl],"username"=>config[:username]}],"displayName"=>config[:displayName],"id"=>echo_id},"itemsPerPage"=>1,"startIndex"=>0,"totalResults"=>1}}}].to_json
    end  
  end
end
